Package: tabularaster
Type: Package
Title: Tidy Tools for 'Raster' Data
Version: 0.7.2
Authors@R: c(person("Michael D.", "Sumner", , "mdsumner@gmail.com", c("aut", "cre")))
Description: Facilities to work with vector and raster data in efficient 
 repeatable and systematic work flow. Missing functionality in existing packages 
 is included here to allow extraction from raster data with 'simple features' and 
 'Spatial' types and to make extraction consistent and straightforward. Extract cell 
 numbers from raster data and  return the cells as a data frame 
 rather than as lists of matrices or vectors. The functions here allow spatial data 
 to be used without special handling for the format currently in use. 
License: GPL-3
LazyData: TRUE
Depends: R (>= 3.2.5)
Imports: dplyr, fasterize, magrittr, raster, silicate, spatstat.geom,
        tibble
Encoding: UTF-8
Suggests: covr, knitr, rmarkdown, testthat (>= 2.1.0), spelling
VignetteBuilder: knitr
URL: https://github.com/hypertidy/tabularaster
BugReports: https://github.com/hypertidy/tabularaster/issues
Language: en-US
NeedsCompilation: no
Packaged: 2023-11-01 05:14:23 UTC; mdsumner
Author: Michael D. Sumner [aut, cre]
Maintainer: Michael D. Sumner <mdsumner@gmail.com>
Repository: CRAN
Date/Publication: 2023-11-01 06:10:03 UTC
Built: R 4.6.0; ; 2026-04-26 22:01:41 UTC; unix
