# Generated by roxygen2: do not edit by hand

S3method(as_tibble,BasicRaster)
S3method(cellnumbers,SpatialLines)
S3method(cellnumbers,default)
S3method(cellnumbers,sf)
S3method(cellnumbers,sfc)
export("%>%")
export(as_tibble)
export(cellnumbers)
export(decimate)
export(index_extent)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,mutate)
importFrom(magrittr,"%>%")
importFrom(raster,"res<-")
importFrom(raster,brick)
importFrom(raster,cellFromLine)
importFrom(raster,cellFromPolygon)
importFrom(raster,cellFromXY)
importFrom(raster,extract)
importFrom(raster,getZ)
importFrom(raster,ncell)
importFrom(raster,nlayers)
importFrom(raster,projection)
importFrom(raster,raster)
importFrom(raster,res)
importFrom(raster,setValues)
importFrom(raster,values)
importFrom(raster,xyFromCell)
importFrom(spatstat.geom,as.owin)
importFrom(spatstat.geom,owin)
importFrom(spatstat.geom,pixellate)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
